@@ -67,7 +67,14 @@ objective.default <- function( x )
 ##' @noRd
 ##' @export
 'objective<-.OP' <- function( x, value ) {
-    x$objective <- as.objective(value)
+    obj <- as.objective(value)
+    nvar <- length(obj)
+    if ( is.na(x[["n_of_variables"]]) ) {
+        x[["n_of_variables"]] <- nvar
+    } else {
+        stopifnot(isTRUE(x[["n_of_variables"]] == nvar))
+    }
+    x[["objective"]] <- obj
     x
 }
 
@@ -113,8 +120,6 @@ length.objective <- function( x ) attr( as.objective(x), "nobj" )
     as.numeric(as.matrix(terms(x)$L[1, i]))
 }
 
-str_default <- function(object, ...) getNamespace("utils")$str.default(object, ...)
-
 ##  NOTE: Since we override the length of the objective the str function which 
 ##        relies on length, doesn't work anymore.
 ##' @noRd
@@ -438,7 +443,8 @@ as.F_objective.function <- function( x ){
     }
     stopifnot( is.function(F), is_integer(n) )
     ans <- tryCatch( F(rep.int(0, n)), error = identity )
-    if( inherits(ans, "error") )
+
+    if( inherits(ans, "error") | (length(ans) != 1L) | !is.finite(ans) )
         stop(sprintf("cannot evaluate function 'F' using 'n' = %d parameters.", n))
     if( !is.numeric(ans) || (length(ans) != 1L) || !is.null(dim(ans)) )
         stop("function 'F' does not return a numeric vector of length 1.")
